// [Licence]
// The Ariba-Underlay Copyright
//
// Copyright (c) 2008-2009, Institute of Telematics, Universität Karlsruhe (TH)
//
// Institute of Telematics
// Universität Karlsruhe (TH)
// Zirkel 2, 76128 Karlsruhe
// Germany
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE INSTITUTE OF TELEMATICS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ARIBA PROJECT OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation
// are those of the authors and should not be interpreted as representing
// official policies, either expressed or implied, of the Institute of
// Telematics.
// [Licence]

#ifndef __ARIBA_CONTEXT_H
#define __ARIBA_CONTEXT_H

#include "ariba/utility/types/SpoVNetID.h"
#include "ariba/utility/types/NodeID.h"
#include "ariba/communication/BaseCommunication.h"
#include "ariba/overlay/BaseOverlay.h"

using ariba::utility::SpoVNetID;
using ariba::utility::NodeID;
using ariba::communication::BaseCommunication;
using ariba::overlay::BaseOverlay;

namespace ariba {
namespace interface {

class AribaContext {
public:
	AribaContext(BaseCommunication& _basecomm, BaseOverlay& _overlay,
			const SpoVNetID& _spovnetid, const NodeID& _nodeid);
	~AribaContext();

	const SpoVNetID& getSpoVNetID() const;
	const NodeID& getNodeID() const;

	BaseOverlay& getOverlay();
	BaseCommunication& getBaseCommunication();

private:
	SpoVNetID spovnetid;
	NodeID nodeid;

	BaseCommunication& basecomm;
	BaseOverlay& overlay;
};

}} // ariba, interface

#endif //__ARIBA_CONTEXT_H
